// EDGE BLACKJACK (CARD GAME)
// By Coday
// Version 0.9
//
// CREDITS:
// Original script by Doti, published as 'The White Queen' and 'Spank Blackjack' by Banjo.
// 'Edge Blackjack' variation script additions and changes by Coday.
// Standard card deck images by John Fitzgibbon (http://www.jfitz.com/cards/)
// Sexy card deck images are taken from Doti's 'The White Queen' SexScripts upload (original source unknown).
// Asian card deck images found collected online, original source unknown.
// Opponent images sourced from various free web preview galleries.
// Card sounds from www.freesound.org, edited by Banjo from sounds originally recorded by F4ngy.
//
// COPYRIGHT:
// This script, from http://ss.deviatenow.com, is protected by this licence:
// CC BY-NC-ND, see http://creativecommons.org/licenses/by-nc-nd/4.0/
// This copyright refers only to this script itself; it does not include any other included resources (images, music, sounds, etc.) which remain the property of their owners.
//
// SCRIPT RESOURCES
// This contains a list of files included with the script when published to the SexScripts website
/*
 * Resources
 * scripts/edgeblackjack.groovy
 * images/edgeblackjack/*.jpg
 * images/edgeblackjack/*.png
 * sounds/edgeblackjack/*.wav
 */
//

// TODO:
// * Limit return time

//Metadata
setInfos(7, "Edge Blackjack",
	"A Blackjack card game in which you bet edges versus clothes, with a chance of orgasm at the end",
	"Coday",
	"v0.9", 0xFFFFFF,"en",
	["game", "edging", "femaledom", "formale", "forfemale"]);

//Player setup
def player = (loadString("intro.name"))
def playerLivesStart
def playerLivesCurrent
//Opponent setup
def opponentLivesStart
def opponentLivesCurrent
def respectTitle
def clothesList = []
def nrImages = []

//General vars
def quit = false
def turns = 0
def roundPoints = 1
def totalPoints = 0
def loss
def win
def totalWins = 0
def totalLosses = 0
def red = loadInteger("edgeblackjack.red")
def waiting = 10
def doubled = false
def maxRestTime = 30
def minRestTime = 10
def holdEdgeChance = 30
def maxEdgeHoldTime = 30
def gameModeRedress = false
def nextTime = loadInteger("edgeblackjack.not_before")
def startTime = loadInteger("edgeblackjack.launch.lasttime")

//Card Sounds
def playSoundCardShuffle = {
	playBackgroundSound("edgeblackjack/card_shuffle.wav")
}

def playSoundCardDeal = {
	playBackgroundSound("edgeblackjack/card_deal.wav")
}

def doneCards = []

// Begin blackjack functions
def getCard = { 
	def ok = false
	def card = null
	while(!ok) {
		card = getRandom(52)
		ok = !doneCards.contains(card)
	}
	doneCards.add(card)
	if(doneCards.size==52)
		doneCards.clear()
	return card
}

def getScore = { card -> 
	if(card<4)
		return 11
	if(card<40)
		return (int)Math.ceil((card+1)/4)
	return 10
}

def getFullScore = { cards ->
	def res = 0
	for(def c : cards)
		res += getScore(c)
	if(res>21) {
		for(def c : cards)
			if(getScore(c)==11 && res>21)
				res-=10
	}
	return (int)res
}

def getCardName = { card ->
	def values = ["Ace","2","3","4","5","6","7","8","9","10","Jack","Queen","King"]
	def colors = ["Clubs", "Diamonds", "Hearts", "Spades"]
	return values[(int)Math.floor(card/4)]+" of "+colors[card%4]
}

//Display card images
def showCards= { cards -> 
	def width = 540
	def height = 500
	def image = new java.awt.image.BufferedImage(width, height, java.awt.image.BufferedImage.TYPE_4BYTE_ABGR)
	def graphics = image.getGraphics()
	def sampleImage = java.awt.Toolkit.getDefaultToolkit().getImage(
		"images/edgeblackjack/card_0.png")
	def oneWidth = sampleImage.getWidth()
	def oneHeight = sampleImage.getHeight()
	for(def i = 0;i<cards.size;i++)
		for(def j = 0;j<cards[i].size;j++) {
			def x = j * (width - oneWidth) / (Math.max(2,cards[i].size-1))
			def delta = (i*j + i * 5 + j * 7) % 4
			def y = i * (height - oneHeight - 3) / (Math.max(1,cards.size-1)) + delta
			def thisImage = javax.imageio.ImageIO.read(new File("images/edgeblackjack/card_"+cards[i][j]+".png"))
			if(i==0) {
				graphics.rotate(Math.PI, width/2, (oneHeight + 3)/2)
			}
			graphics.drawImage(thisImage, (int)x, (int)y, null)
			if(i==0) {
				graphics.rotate(-Math.PI, width/2, (oneHeight + 3)/2)
			}
		}
	java.io.ByteArrayOutputStream baos = new java.io.ByteArrayOutputStream()
	javax.imageio.ImageIO.write(image, "png", baos)
	setImage(baos.toByteArray(),0)
}

//Blackjack hand
def playBlackjackHand = {
	def cards = [[],[]]
	cards[1].add(getCard())
	playSoundCardDeal()
	showCards(cards)
    roundPoints = 1 + opponentLivesStart - opponentLivesCurrent
    def ante = "I bet my " + clothesList[opponentLivesCurrent -1] + "\nYou bet " + roundPoints + " edges\n\n"
	show(ante + "Your first card: "+getCardName(cards[1][0])+".\n\n"+
		"My hand: "+getFullScore(cards[0])+"\n"+
		"Your hand: "+getFullScore(cards[1])+"")
	wait(waiting)
	cards[0].add(getCard())
	playSoundCardDeal()
	showCards(cards)
	show(ante + "My first card: "+getCardName(cards[0][0])+".\n\n"+
		"My hand: "+getFullScore(cards[0])+"\n"+
		"Your hand: "+getFullScore(cards[1])+"")
	wait(waiting)
	cards[1].add(getCard())
	playSoundCardDeal()
	showCards(cards)
	show(ante + "Your second card: "+getCardName(cards[1][1])+".\n\n"+
		"My hand: "+getFullScore(cards[0])+"\n"+
		"Your hand: "+getFullScore(cards[1])+"")
	wait(waiting)
	doubled = false
	def passed = false
	while(getFullScore(cards[1])<21 && !passed && !doubled) {
		def choice = -1
		if(cards[1].size==2)
			choice = getSelectedValue(ante + "What do you want to do?\n\n"+
				"My hand: "+getFullScore(cards[0])+"\n"+
				"Your hand: "+getFullScore(cards[1])+"", ["Hit", "Stand", "Double"])
		else	
			choice = getSelectedValue(ante + "What do you want to do?\n\n"+
				"My hand: "+getFullScore(cards[0])+"\n"+
				"Your hand: "+getFullScore(cards[1])+"", ["Hit", "Stand"])
		passed = choice==1
		if(choice==2) {
			doubled = true
            roundPoints = roundPoints * 2
            if ( opponentLivesCurrent > 1 )
			    ante = "I bet my " + clothesList[opponentLivesCurrent -1] + " and my " + clothesList[opponentLivesCurrent -2] + 
					"\nYou bet " + roundPoints + " edges\n\n"
            else
                ante = "I bet my " + clothesList[opponentLivesCurrent -1] + "\nYou bet " + roundPoints + " edges\n\n"
		}
		if(choice==0 || choice==2) {
			cards[1].add(getCard())
			playSoundCardDeal()
			showCards(cards)
			show(ante+"Your card: "+getCardName(cards[1][cards[1].size-1])+".\n\n"+
				"My hand: "+getFullScore(cards[0])+"\n"+
				"Your hand: "+getFullScore(cards[1])+"")
			wait(waiting)
		}
	}


	if(getFullScore(cards[1])>21) {
		show(ante+"You bust! You lose this hand.\n\n"+
			"My hand: "+getFullScore(cards[0])+"\n"+
			"Your hand: "+getFullScore(cards[1])+"")
			loss = 1
			win = -1
	} else if (getFullScore(cards[1])==21 && cards[1].size() == 2 ) {
		// Player has blackjack - now is an immediate win
		show("You have Blackjack! You win this hand.\nYour chance at an orgasm at the end increases.")
		if ( red > 0 ) {
			red--
			save("edgeblackjack.red", red)
		}
		loss = -1
		win = 1
	} else {
		cards[0].add(getCard())
		playSoundCardDeal()
		showCards(cards)
		show(ante+"My second card: "+getCardName(cards[0][1])+".\n\n"+
			"My hand: "+getFullScore(cards[0])+"\n"+
			"Your hand: "+getFullScore(cards[1])+"")
		wait(waiting)
		while(getFullScore(cards[0])<17 && getFullScore(cards[0]) <= getFullScore(cards[1])) {
			cards[0].add(getCard())
			playSoundCardDeal()
			showCards(cards)
			show("My card: "+getCardName(cards[0][cards[0].size-1])+".\n\n"+
				"My hand: "+getFullScore(cards[0])+"\n"+
				"Your hand: "+getFullScore(cards[1])+"")
			wait(waiting * 0.8)
		}
		if(getFullScore(cards[0]) == 21 && cards[0].size == 2) {
			// Opponent has blackjack
			show("I have Blackjack! I win this hand.\n\nYour chance at an orgasm at the end decreases.")
			red++
			save("edgeblackjack.red", red)
			loss = 1
			win = -1
		} else if(getFullScore(cards[0])>21) {
			show(ante+"I bust! You win this hand.\n\n"+
				"My hand: "+getFullScore(cards[0])+"\n"+
				"Your hand: "+getFullScore(cards[1])+"")
				loss = -1
				win = 1
		} else if(getFullScore(cards[1])==getFullScore(cards[0])) {
			show(ante+"Push. No winner this hand.\n\n"+
			"My hand: "+getFullScore(cards[0])+"\n"+
			"Your hand: "+getFullScore(cards[1])+"")
			loss = 0
			win = 0
		} else if(getFullScore(cards[1])>getFullScore(cards[0])) {
			show("You win this hand.\n\n"+
			"My hand: "+getFullScore(cards[0])+"\n"+
			"Your hand: "+getFullScore(cards[1])+"")
			loss = -1
			win = 1
		} else {
			show("You lose the hand.\n\n"+
			"My hand: "+getFullScore(cards[0])+"\n"+
			"Your hand: "+getFullScore(cards[1])+"")
			loss = 1
			win = -1
		}
	}
	showButton("Continue.")
}
//Rules explaination
def rulesDisplay = {
	if(!getBoolean("Do you know how to play regular Blackjack, "+player+"?")) {
		show("That doesn't say much for your chances, I'm afraid.\n\nAlright, the basic rules:\n\n"+
			"To win a hand, the combined value of your cards must be as close to 21 as possible, but no more.\n\n"+
			"A face card (Jack, Queen, King) is worth 10.\n\n"+
			"An Ace is worth 1 or 11.\n\n"+
			"If you have a face card and an Ace, that's Blackjack... it's an automatic win!")
		showButton("I understand, "+respectTitle+".")
	}
	show("For this game, I'll be the dealer."
		+"\n\nIf I win a hand, you must edge a number of times, depending on how much clothes I'm still wearing"
		+"\nIf you win a hand, I will remove one item of clothing."
		+"\nIf you get me completely naked, You will have to do some more edges."
		+"\nAfter the last edge I will decide your fate by drawing a bead from a bag of beads."
		+"\nThere will be a number of red beads and one green bead in that bag."
		+"\nIf I draw the green bead, you will be granted an orgasm."
		+"\nHowever if I draw a red bead, your orgasm is denied."
		+"\n\nThe number of red beads will be saved so your chance of an orgasm increases with each won game."
		+"\n\nUnderstand?")
	showButton("Yes, "+respectTitle+".")
	show("We will play with 2 decks, with no insurance or splitting allowed."
		+"\n\nIf I win a hand with blackjack, I will put another red bead in the bag."
		+"\n\nIf you win the hand with blackjack I will remove a red bead from the bag."
		+"\n\nYou may double down to double the number of clothes I get to remove."
		+"\n\nYou will have to do double the number of edges if I win the hand!")
	showButton("Yes, "+respectTitle+".")
}



//End of round
def endRound = {
	//Set stat report format according to game mode
	def midroundMessageReport = "My Clothes Remaining: "+opponentLivesCurrent

	//Set taunt message and whether points were earned or reduced
	def midroundMessageArray
	def midroundMessagePoints
	//Opponent won round
	if(win==-1) {
		midroundMessageArray = ["Are you feeling desperate yet, "+player+"?",
			"I love to see you squrim!",
			"The house wins.",
			"Beating you is so much fun!",
			"You don't stand a chance, "+player+".",
			"You're never going to cum if you keep this up!",
			"You're not very good at this game, are you?",
			"This is almost too easy!"]
	}
	//Player won round
	else if(win==1) {
		midroundMessageArray = ["I can't believe you won that hand!",
			"Enjoy your victory while you can, "+player+".",
			"You'd better not be cheating!",
			"You got lucky that time.",
			"That was pure luck!",
			"Noooo!",
			"Seriously?!",
			"I think there's something wrong with these cards..."]
	}
	//Round was a draw
	else {
		midroundMessageArray = ["Shall we continue?",
			"Enjoying yourself?",
			"This is so much fun!",
			"The excitement is building!",
			"Isn't this fun, "+player+"?",
			"Intense, huh?",
			"I like it when you put up a struggle.",
			"I love this game!"]
	}
	//Display game progress and taunt
	show(""+midroundMessageReport+"\n\n\n"+midroundMessageArray[getRandom(8)]+"")
	showButton("Continue")
	//Reset round variables
	win = 0
	loss = 0
	//Increment turn
	turns++
}
//Set correct opponent image
def setOpponentImage = {
	def nr = getRandom(nrImages[opponentLivesCurrent])
    setImage( "edgeblackjack/op_" + opponentLivesCurrent + "_" + nr + ".jpg" )
//	setImage( "edgeblackjack/cat0001.jpg" )
}
//Adjust lives and win/losses for player and opponent
def undress = {
	if ( opponentLivesCurrent > 0 ) {
		opponentLivesCurrent = opponentLivesCurrent - 1
        show("Looks like I get to remove my " + clothesList[opponentLivesCurrent] + "." )
		setOpponentImage()
		wait(3)
    }
}

def redress = {
	if ( opponentLivesCurrent < opponentLivesStart ) {
		show("Seems like I get to put my " + clothesList[opponentLivesCurrent ] + " back on...")
		opponentLivesCurrent = opponentLivesCurrent + 1
		setOpponentImage()
		wait(3)
	}
}

def doEdge = { 
	show( "Edge for me...." )
	showButton("I'm on the edge, " + respectTitle +".")

	// add a chance to hold the edge
	if (getRandom(100) < holdEdgeChance ) {
		show( "Hold it......." )
		waitWithGauge(getRandom(maxEdgeHoldTime) + 1)
	}
}

//Adjust clothing and show dialog for round winner and loser if stripping
def roundWinner = {
	//Opponent won
    if (gameModeRedress && win==-1) {
		redress()
		if ( doubled )
			redress()
	}
	//Opponent lost
	else if(win==1)	{
		undress()
		if ( doubled )
			undress()
	}
	setOpponentImage()	
	//Player lost
	if(loss==1) {
		for(def i=roundPoints-1;i >= 0;i--) {
			doEdge()
			show( "Hands off......" )
			wait(2)
			if ( i > 0 ) {
				show( "Take a short break to recover from that edge, " + player + "!")
				waitWithGauge(getRandom(maxRestTime-minRestTime) + minRestTime)
			}
			setOpponentImage()
		}
	}
}

//Game is over
def endGame = {
	show( "Congratulations, " + player + "!\n\nYou got me completely naked, now let's see if you get to cum today.....")
	wait(2)
	// do some Edges
	for(def i=getRandom(5);i >= 0;i--) {
		doEdge()
		show( "Hands off......" )
		wait(2)
		if ( i > 0 ) {
			show( "Take a short break to recover from that edge, " + player + "!")
			waitWithGauge(getRandom(maxRestTime-minRestTime) + minRestTime)
		}
		setOpponentImage()
	}
	// decide fate
	doEdge()
	if (getRandom(red+1) == 0 ) {
		setImage("edgeblackjack/op_bonus.jpg")
		show("Cum for me, " + player + "!")
		red = red + 6
	} else {
		setOpponentImage()
		show( "Hands off, " + player+ "!\n\nYour orgasm is denied!")
		if ( red > 0 )
			red--
	}
	save("edgeblackjack.red", red)
	showButton( "Continue" )
}

//Initialize game
def init = {
	playerLivesStart = 1
    clothesList = [ "stockings", "panties", "bra", "skirt", "blouse", "jacket" ]
    nrImages = [ 7, 8, 8, 5, 7, 5, 7 ]
    opponentLivesStart = clothesList.size()
    respectTitle = "Miss"
	if ( red == null )
		red = 6
}

//Setup clothes/lives for player and opponent
def setupLives = {
	playerLivesCurrent = playerLivesStart
	opponentLivesCurrent = opponentLivesStart
}

	
//Start of script
init()
setupLives()
setOpponentImage()
// check if player is back too soon
if ( nextTime != null && startTime < nextTime ) {
	show("Welcome back, " + player + "!\n\nI knew you would be eager to come back and play again.\n\nYou are back too soon though.\n\nYou may only play once per 12 hours!")
	quit = true
	showButton("Goodbye, "+respectTitle+".")
} else {
	// Temporarily add debug info
	nextTime = startTime + (3600 * 12)
	save( "edgeblackjack.not_before", nextTime )
}

while(!quit) {
	if(getBoolean("Do you want to review the rules?","Yes","I know them"))
	{
		rulesDisplay()
		while(getBoolean("Do you need to go back over the rules again?","Yes","I understand them"))
		{
			rulesDisplay()
		}
	}
	show("Excellent, then let's play some Blackjack!")
	wait(3)
	setImage(null)
	show(null)
	turns = 0
	while(opponentLivesCurrent>0) {
		waiting = 2 + (2 / (1+turns/4))
		playBlackjackHand()
		roundWinner()
		endRound()
	}
	endGame()
	show("Goodbye, "+player +".")
	showButton("Goodbye, "+respectTitle+".")
	quit = true
}
setImage(null)
show(null)
def quitGame = true

if(quitGame) {
	return null
}

//END OF SCRIPT
